/* 
   ScanJetPlay
   (c) 2005 by GanjaTron (ganjatron@gmx.net)   
   
   Plays tune on HP Scanjet 3p/3c/4p/4c (others?)
   
   Song files consist of 3 bytes / note:

   <freqMSB> <freqLSB> <duration> ...,

   where the frequency specifies the number of 1.5 MHz clock cycles between
   motor half-steps, i.e. note frequency = 1.5 MHz / (freqMSB * 256 +
   freqLSB) The duration is specified as multiples of ca. 1/8 sec.
   
   The song length is prepended automatically before uploading to the
   scanner.
   
   See also: S.L. Webb, K.J. Youngers, M.J. Steinle, and J.A. Eccher,
   "Design of a 600-Pixel-per-Inch, 30-Bit Color Scanner", HP Journal 2/97.
      
   -----------------------------------------------------------------------
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; use at your own risk. The author assumes no
   liability for any damages incurred by its use, particularly to scanning
   hardware. Care has been taken to avoid potential damage, but if your
   scanner makes strange noises SHUT IT DOWN IMMEDIATELY!.

   By using this software, you agree to these terms and conditions.
   -----------------------------------------------------------------------
*/



#define MAXBUFSIZE (1U << 16)
#define PLAYTUNE "\033*u0M"
#define RESET "\033E"



#include <stdio.h>
#include <string.h>
#include "sane/sanei_scsi.h"



int main (int argc, char* argv[])
{
   SANE_Status stat;
   int fd;
   size_t dataLen;
   FILE *songFile;
   char scsiCmd [10], scsiData [MAXBUFSIZE];
   
   if (argc < 3) {
      fprintf(stderr, "%s <deviceName> <songFile>\n", argv [0]);
      return 1;
   }
   /* Open & read song file */
   songFile = fopen(argv [2], "rb");
   if (!songFile) {
      fprintf(stderr, "Failed to open song file %s, aborting\n", argv [2]);
      return 1;
   }
   dataLen = fread(scsiData + 2, 1, MAXBUFSIZE, songFile) / 3;
   fclose(songFile);
   if (!dataLen) {
      fputs("Corrupt song file, aborting\n", stderr);
      return 1;
   }
   /* ScanJet apparently uses a zero-based note counter */
   scsiData [0] = dataLen - 1 >> 8;
   scsiData [1] = dataLen - 1;
   dataLen = dataLen * 3 + 2;
   /* Open scanner device */
   stat = sanei_scsi_open(argv [1], &fd, NULL, NULL);
   if (stat != SANE_STATUS_GOOD) {
      fprintf(stderr, "Failed to open device %s, aborting\n", argv [1]);
      return 1;
   }
   /* Upload notes via SCSI WRITE BUFFA command */
   memset(scsiCmd, 0, 10);
   scsiCmd [0] = 0x3b;
   scsiCmd [1] = 2;
   scsiCmd [6] = dataLen >> 16;
   scsiCmd [7] = dataLen >> 8;
   scsiCmd [8] = dataLen;
   stat = sanei_scsi_cmd2(fd, scsiCmd, 10, scsiData, dataLen, NULL, NULL);
   if (stat != SANE_STATUS_GOOD) {
      fputs("SCSI WRITE BUFFER command failed, aborting\n", stderr);
      sanei_scsi_close(fd);
      return 1;
   }
   /* Send PLAY TUNE command */
   memset(scsiCmd, 0, 6);
   scsiCmd [0] = 0x0a;
   scsiCmd [4] = dataLen = strlen(PLAYTUNE);
   strcpy(scsiData, PLAYTUNE);
   stat = sanei_scsi_cmd2(fd, scsiCmd, 6, scsiData, dataLen, NULL, NULL);
   if (stat != SANE_STATUS_GOOD) {
      fputs("PLAY TUNE command failed, aborting\n", stderr);
      sanei_scsi_close(fd);
      return 1;
   }
   /* Send RESET command to return carriage and avoid potential disaster */
   memset(scsiCmd, 0, 6);
   scsiCmd [0] = 0x0a;
   scsiCmd [4] = dataLen = strlen(RESET);
   strcpy(scsiData, RESET);
   stat = sanei_scsi_cmd2(fd, scsiCmd, 6, scsiData, dataLen, NULL, NULL);
   if (stat != SANE_STATUS_GOOD) {
      fputs("RESET failed, power down scanner!\n", stderr);
      sanei_scsi_close(fd);
      return 1;
   }   
   sanei_scsi_close(fd);
   return 0;
}
